# Generated by roxygen2: do not edit by hand

S3method(plot,bppr)
S3method(predict,bppr)
S3method(predict,bppr_pca)
S3method(print,bppr)
S3method(summary,bppr)
export(bppr)
export(bppr_pca)
export(bppr_resume)
import(graphics)
import(stats)
import(utils)
