# BayesPPR

<!-- badges: start -->
[![R-CMD-check](https://github.com/gqcollins/BayesPPR//actions/workflows/r.yml/badge.svg)](https://github.com/gqcollins/BayesPPR//actions/workflows/r.yml)
[![CRAN status](https://www.r-pkg.org/badges/version/BayesPPR)](https://CRAN.R-project.org/package=BayesPPR)
<!-- badges: end -->


## Description

BayesPPR is an R package used for fitting a Bayesian Projection Pursuit Regression model. (Collins et al., 2023 <DOI:10.1007/s11222-023-10334-z>)

### Installation
------------------------------------------------------------------------------
v0.1.0 is on [CRAN](https://cran.r-project.org/package=BayesPPR) and can
be installed as

``` r
install.packages("BayesPPR")`
```

For a more up to date, but may not be stable version from git
repository.

1.  Download zip or tar.gz of package or clone repository
2.  Install into R (\> 4.3.0)

``` r
library(devtools)
install_github("gqcollins/BayesPPR")
```

------------------------------------------------------------------------------


## Attribution

Copyright 2022 for C21079 BayesPPR 

This program is open source under the BSD-3 License. Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of conditions and
the following disclaimer.
 
2.Redistributions in binary form must reproduce the above copyright notice, this list of conditions
and the following disclaimer in the documentation and/or other materials provided with the
distribution.
 
3.Neither the name of the copyright holder nor the names of its contributors may be used to endorse
or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


© 2022. Triad National Security, LLC. All rights reserved. This program was produced under U.S. Government contract 89233218CNA000001 for Los Alamos National Laboratory (LANL), which is operated by Triad National Security, LLC for the U.S. Department of Energy/National Nuclear Security Administration. All rights in the program are reserved by Triad National Security, LLC, and the U.S. Department of Energy/National Nuclear Security Administration. The Government is granted for itself and others acting on its behalf a nonexclusive, paid-up, irrevocable worldwide license in this material to reproduce, prepare derivative works, distribute copies to the public, perform publicly and display publicly, and to permit others to do so.
